
#include "actor-destructible.h"



ActorDestructible::ActorDestructible()
{
	type = "ActorDestructible";
	hit_points = 100;
	idle_anim = NULL;
	death_anim = NULL;
	dead = false;
	hurt_as_player = false;
	hurt_as_enemy = false;
	death_behavior = NULL;
}



ActorDestructible::~ActorDestructible()
{
	if (strcmp(type, "ActorDestructible") == 0) {
		if (idle_anim != NULL) delete idle_anim;
		if (death_anim != NULL) delete death_anim;
		if (death_behavior != NULL) delete death_behavior;
	}
}



Actor::ActorSignal ActorDestructible::update()
{
	ActorSignal retsignal = {"", this};

	if (hit_points > 0) set_anim(idle_anim);
	if (hit_points <= 0) {
		dead = true;
		set_anim(death_anim);
		if (death_behavior != NULL) {
			if (get_behavior() != NULL) delete get_behavior();
			set_behavior(death_behavior);
			set_death_behavior(NULL);
		}
		if (death_anim->at_end()) {
			retsignal.signal = "suicide";
			retsignal.source = this;
		}
	}
	if (get_death_on_leaving_scope() && !get_scope().contains(get_bounding_box())) {
		retsignal.signal = "suicide";
		retsignal.source = this;
	}
	update_base();
	return retsignal;
}



void ActorDestructible::get_signal(ActorSignal signal)
{
	if ((strcmp(signal.signal, "hurt") == 0) && !dead) {
		if (signal.source->damages_players() && gets_hurt_as_player()) {
			hurt(signal.source->get_collision_damage());
		} else if (signal.source->damages_enemies() && gets_hurt_as_enemy()) {
			hurt(signal.source->get_collision_damage());
		}		
	}
}



void ActorDestructible::set_hp(int hp)
{
	hit_points = hp;
}



void ActorDestructible::hurt(int hp)
{
	hit_points -= hp;
}



void ActorDestructible::heal(int hp)
{
	hit_points += hp;
}



int ActorDestructible::get_hp()
{
	if (hit_points > 0)
		return hit_points;
	return 0;
}



void ActorDestructible::set_death_anim(Anim *new_anim)
{
	death_anim = new_anim;
}



Anim *ActorDestructible::get_death_anim()
{
	return death_anim;
}



void ActorDestructible::set_idle_anim(Anim *new_anim)
{
	idle_anim = new_anim;
	if (get_anim() == NULL)
		set_anim(idle_anim);
}



void ActorDestructible::kill()
{
	hit_points = 0;
}



void ActorDestructible::set_hurt_types(bool players, bool enemies)
{
	hurt_as_player = players;
	hurt_as_enemy = enemies;
}



bool ActorDestructible::gets_hurt_as_player()
{
	return hurt_as_player;
}



bool ActorDestructible::gets_hurt_as_enemy()
{
	return hurt_as_enemy;
}



Actor::ActorSignal ActorDestructible::collided()
{
	ActorSignal signal = {"", NULL};
	if (!dead) {
		signal.signal = "hurt";
		signal.source = this;
	}
	
	return signal;
}



void ActorDestructible::set_death_behavior(Behavior *new_death_behavior)
{
	death_behavior = new_death_behavior;
}



Behavior *ActorDestructible::get_death_behavior()
{
	return death_behavior;
}



Anim *ActorDestructible::get_idle_anim()
{
	return idle_anim;
}

