
#include "actor-player.h"



ActorPlayer::ActorPlayer(Event *event)
{
	type = "ActorPlayer";
	set_hp(100);
	score = 0;
	set_damage_types(false, true);
	set_collision_damage(1);
	to_next_weapon_switch = 0;
	to_next_yelp = 0;
	facing = demon_forward;
	for (int count = 1; count <= NUM_WEAPONS; count++) {
		to_next_shot[count] = 0;
		weapon_power[count] = 1;
	}
	weapon_name[1] = "Firebolt";
	weapon_name[2] = "Lightning";
	weapon_name[3] = "Minibolt";
	weapon_power[1] = 1;
	weapon_power[2] = 1;
	weapon_power[3] = 1;
	current_weapon = 1;
	
	Anim *temp = new Anim;
	
	temp->add_frame("demonforetoback00", 5);
	temp->add_frame("demonforetoback01", 5);
	temp->add_frame("demondeath00", 3);
	temp->add_frame("demondeath01", 3);
	temp->add_frame("demondeath02", 3);
	temp->add_frame("demondeath03", 3);
	temp->add_frame("demondeath04", 3);
	temp->add_frame("demondeath05", 3);
	temp->add_frame("demondeath06", 3);
	temp->add_frame("demondeath07", 3);
	temp->add_frame("demondeath08", 3);
	temp->add_frame("demondeath09", 5);
	temp->add_frame("demondeath10", 5);
	temp->add_frame("demondeath11", 5);
	temp->set_looping(false);
  temp->add_hash();
		
	set_death_anim(temp);
	
	wings_anim = new Anim;
	wings_anim->add_frame("demonwings00", 10);
	wings_anim->add_frame("demonwings01", 6);
	wings_anim->add_frame("demonwings02", 10);
	wings_anim->add_frame("demonwings01", 6);
	wings_anim->add_hash();
	wings_anim->set_looping(true);
	forward_anim = new Anim;
	forward_anim->add_frame("demonforward00", 25);
	forward_anim->add_hash();
	forward_anim->set_looping(true);
	forward_shoot_anim = new Anim;
	forward_shoot_anim->add_frame("demonforward02", 6);
	forward_shoot_anim->add_frame("demonforward01", 6);
	forward_shoot_anim->add_hash();
	forward_shoot_anim->set_looping(false);
	forward_blue_anim = new Anim;
	forward_blue_anim->add_frame("demonforward06", 8);
	forward_blue_anim->add_frame("demonforward01", 8);
	forward_blue_anim->add_hash();
	forward_blue_anim->set_looping(false);
	forward_super_shoot_anim = new Anim;
	forward_super_shoot_anim->add_frame("demonforward05", 8);
	forward_super_shoot_anim->add_frame("demonforward04", 8);
	forward_super_shoot_anim->add_frame("demonforward05", 8);
	forward_super_shoot_anim->add_frame("demonforward03", 5);
	forward_super_shoot_anim->set_looping(false);
	forward_super_shoot_anim->add_hash();
	backward_anim = new Anim;
	backward_anim->add_frame("demonbackward00", 25);
	backward_anim->add_hash();
	backward_anim->set_looping(true);
	backward_shoot_anim = new Anim;
	backward_shoot_anim->add_frame("demonbackward01", 6, 34);
	backward_shoot_anim->add_frame("demonbackward02", 6, 34);
	backward_shoot_anim->add_hash();
	backward_shoot_anim->set_looping(false);
	backward_blue_anim = new Anim;
	backward_blue_anim->add_frame("demonbackward06", 8, 34);
	backward_blue_anim->add_frame("demonbackward02", 8, 34);
	backward_blue_anim->add_hash();
	backward_blue_anim->set_looping(false);
	backward_super_shoot_anim = new Anim;
	backward_super_shoot_anim->add_frame("demonbackward05", 8, 34);
	backward_super_shoot_anim->add_frame("demonbackward04", 8, 34);
	backward_super_shoot_anim->add_frame("demonbackward05", 8, 34);
	backward_super_shoot_anim->add_frame("demonbackward03", 5, 34);
	backward_super_shoot_anim->add_hash();
	backward_super_shoot_anim->set_looping(false);
	fore_to_back_anim = new Anim;
	fore_to_back_anim->add_frame("demonforetoback00", 2, 7, 0, -24, -10);
	fore_to_back_anim->add_frame("demonforetoback01", 2, 21, 0, -28, -3);
	fore_to_back_anim->add_frame("demonforetoback02", 2, 30, 0, -13, -2);
	fore_to_back_anim->add_hash();
	fore_to_back_anim->set_looping(false);
	back_to_fore_anim = new Anim;
	back_to_fore_anim->add_frame("demonforetoback02", 2);
	back_to_fore_anim->add_frame("demonforetoback01", 2, 21, 0, -28, -3);
	back_to_fore_anim->add_frame("demonforetoback00", 2, 7, 0, -24, -10);
	back_to_fore_anim->add_hash();
	back_to_fore_anim->set_looping(false);
	set_anim(forward_anim);
	BehaviorFromKeyboard *be = new BehaviorFromKeyboard;
	be->set_bounds(0,0, 220, 210);
	be->bind_event_handler(event);
	set_behavior(be);
}



ActorPlayer::~ActorPlayer()
{
	delete get_death_anim();
	delete wings_anim;
	delete forward_anim;
	delete forward_shoot_anim;
	delete forward_super_shoot_anim;
	delete forward_blue_anim;
	delete backward_blue_anim;
}



Actor::ActorSignal ActorPlayer::update()
{
 	ActorSignal retsignal = {"", this};
  Behavior *temp;
 	int temp_x, temp_y;
 	int count;
 	char *behavior_signal;
 	
 	if (to_next_yelp > 0)
 		to_next_yelp--;
 	if (to_next_weapon_switch > 0)
 		to_next_weapon_switch--;
 	
 	//std::cerr << ((BehaviorFromKeyboard *)get_behavior())->get_dx();
 	
 	if (((BehaviorFromKeyboard *)get_behavior())->get_dx() < -1)
 		facing = demon_backward;
 	else
 		if (((BehaviorFromKeyboard *)get_behavior())->get_dx() > 1)
 			facing = demon_forward;
 	
 	if ((((get_anim() == forward_anim) || ((get_anim() == forward_shoot_anim) || (get_anim() == forward_super_shoot_anim)))
 		|| (get_anim() == forward_super_shoot_anim) || get_anim() == forward_blue_anim) && (facing == demon_backward)) {
 		fore_to_back_anim->reset();
 		set_anim(fore_to_back_anim);
 	} else {
	 	if ((((get_anim() == backward_anim) || ((get_anim() == backward_shoot_anim) || (get_anim() == backward_super_shoot_anim)))
	 		|| (get_anim() == backward_super_shoot_anim) || get_anim() == backward_blue_anim) && (facing == demon_forward)) {
	 		back_to_fore_anim->reset();
	 		set_anim(back_to_fore_anim);
	 	}
	}
 	
 	for (count = 1; count <= NUM_WEAPONS; count++)
 		if (to_next_shot[current_weapon] > 0)
	 		to_next_shot[current_weapon] -= 1;
	if (get_anim()->at_end() && (get_anim() != get_death_anim())) {
		if (facing == demon_forward)
			set_anim(forward_anim);
		if (facing == demon_backward)
			set_anim(backward_anim);
	}
	 	
	if (get_hp() <= 0) {
		if (dead == false) {
			temp = get_behavior();
			delete temp;
			temp = new BehaviorLinear;
			set_behavior(temp);
			set_anim(get_death_anim());
		}
		dead = true;
		if (get_death_anim()->at_end()) {
			retsignal.signal = "suicide";
			retsignal.source = this;
		}
	}
	if (strcmp(get_death_anim()->get_current_frame(), "demondeath00") == 0)
		wings_anim->clear();
	wings_anim->update();
	temp_x = get_x();
	temp_y = get_y();
	if (get_behavior() != NULL)
		behavior_signal = get_behavior()->update(temp_x, temp_y);
	if (strcmp(behavior_signal, "fire") == 0)
		fire();
	if (strcmp(behavior_signal, "prev_weapon") == 0)
		next_weapon();
	if (strcmp(behavior_signal, "next_weapon") == 0)
		prev_weapon();
	set_position(temp_x, temp_y);
	if (get_anim() != NULL) {
		get_anim()->update();
		update_bounding_box();
	}
	return retsignal;
}



// Tell the display to paint the actor.
void ActorPlayer::paint()
{
	if (get_display() == NULL) {
		std::cerr << "WARNING: tried to paint an ActorPlayer before display bound" << std::endl;
	} else if (get_anim() == NULL) {
		std::cerr << "WARNING: tried to paint an ActorPlayer without an anim" << std::endl;
	} else {
		get_display()->paint_sprite(get_x(), get_y(), get_anim()->get_current_frame());
		get_display()->paint_sprite(get_x(), get_y(), wings_anim->get_current_frame());
	}
}



void ActorPlayer::fire()
{
	int offset_y;
	if (facing == demon_backward) {
		offset_y = -5;
	} else {
		offset_y = 0;
	}
	if (to_next_shot[current_weapon] == 0) {
		forward_shoot_anim->reset();
		switch (current_weapon) {
			case 1:
				get_mixer()->play_sound("minibolt");
				if (weapon_power[1] >= 1) {
					if (weapon_power[1] == 1) {
						if (facing == demon_backward && ((get_anim() == backward_anim) || (get_anim() == backward_shoot_anim))) {
							backward_shoot_anim->reset();
							set_anim(backward_shoot_anim);
						} else {
							if (get_anim() == forward_anim || get_anim() == forward_shoot_anim) {
								forward_shoot_anim->reset();
								set_anim(forward_shoot_anim);
							}
						}
					}
					to_next_shot[current_weapon] = 40;
					attack_in_progress = false;
					BulletFirebolt *bullet = new BulletFirebolt;
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 47, get_y() + 7 + offset_y);
				}
				if (weapon_power[1] >= 2) {
					if (facing == demon_backward && (get_anim() == backward_anim || get_anim() == backward_super_shoot_anim)) {
						backward_super_shoot_anim->reset();
						set_anim(backward_super_shoot_anim);
					} else {
						if (get_anim() == forward_anim || get_anim() == forward_super_shoot_anim) {
							forward_super_shoot_anim->reset();
							set_anim(forward_super_shoot_anim);
						}
					}
					BulletFirebolt *bullet = new BulletFirebolt;
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 47, get_y() + offset_y);
					bullet = new BulletFirebolt;
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 47, get_y() + 14 + offset_y);
				}
				if (weapon_power[1] == 3) {
					BulletFirebolt *bullet = new BulletFirebolt(-.3);
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 47, get_y() + offset_y);
					bullet = new BulletFirebolt(.3);
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 47, get_y() + 14 + offset_y);
				}
				break;
			case 2:
				get_mixer()->play_sound("crack");
				if (weapon_power[2] >= 1) {
					if (weapon_power[2] == 1) {
						if (facing == demon_backward && (get_anim() == backward_anim || get_anim() == backward_shoot_anim || get_anim() == backward_blue_anim)) {
							backward_blue_anim->reset();
							set_anim(backward_blue_anim);
						} else {
							if (get_anim() == forward_anim || get_anim() == forward_shoot_anim || get_anim() == forward_blue_anim) {
								forward_blue_anim->reset();
								set_anim(forward_blue_anim);
							}
						}
					}
					to_next_shot[current_weapon] = 65;
					attack_in_progress = false;
					BulletLightning *bullet = new BulletLightning;
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 47, get_y() + 7 + offset_y);
				}
				break;
			case 3:
				get_mixer()->play_sound("shot");
				if (weapon_power[3] >= 1) {
					if (weapon_power[3] == 1) {
						if (facing == demon_backward && ((get_anim() == backward_anim) || (get_anim() == backward_shoot_anim))) {
							backward_shoot_anim->reset();
							set_anim(backward_shoot_anim);
						} else {
							if (get_anim() == forward_anim || get_anim() == forward_shoot_anim) {
								forward_shoot_anim->reset();
								set_anim(forward_shoot_anim);
							}
						}
					}
					to_next_shot[current_weapon] = 5;
					attack_in_progress = false;
					BulletMinibolt *bullet = new BulletMinibolt;
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 36, get_y() + 11 + offset_y);
				}
				if (weapon_power[3] >= 2) {
					if (facing == demon_backward && (get_anim() == backward_anim || get_anim() == backward_super_shoot_anim)) {
						backward_super_shoot_anim->reset();
						set_anim(backward_super_shoot_anim);
					} else {
						if (get_anim() == forward_anim || get_anim() == forward_super_shoot_anim) {
							forward_super_shoot_anim->reset();
							set_anim(forward_super_shoot_anim);
						}
					}
					BulletMinibolt *bullet = new BulletMinibolt(-.5);
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 36, get_y() + 7 + offset_y);
					bullet = new BulletMinibolt(.5);
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 36, get_y() + 13 + offset_y);
				}
				if (weapon_power[3] == 3) {
					BulletMinibolt *bullet = new BulletMinibolt(-1);
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 36, get_y() + 3 + offset_y);
					bullet = new BulletMinibolt(1);
					bullet->set_owner(this);
					get_actorlist()->add_actor(bullet);
					bullet->set_position(get_x() + 36, get_y() + 17 + offset_y);
				}
				break;
		}
	}
}



void ActorPlayer::get_signal(ActorSignal signal)
{
	Enemy *enemy;

	if ((strcmp(signal.signal, "hurt") == 0) && signal.source->damages_players() == true) {
		if (to_next_yelp == 0 && !dead) {
			get_mixer()->play_sound("yelp");
			to_next_yelp = PLAYER_YELP_DELAY;
		}
		hurt(signal.source->get_collision_damage());
	}
	if (strcmp(signal.signal, "enemy_killed") == 0) {
		enemy = (Enemy *)signal.source;
		add_to_score(enemy->get_point_value());
	}
}



void ActorPlayer::prev_weapon()
{
	if (to_next_weapon_switch <= 0) {
		current_weapon--;
		if (current_weapon <= 0) current_weapon = NUM_WEAPONS;
		to_next_weapon_switch = WEAPON_SWITCH_SPEED;
	}
}



void ActorPlayer::next_weapon()
{
	if (to_next_weapon_switch <= 0) {
		current_weapon++;
		if (current_weapon > NUM_WEAPONS) current_weapon = 1;
		to_next_weapon_switch = WEAPON_SWITCH_SPEED;
	}
}



int ActorPlayer::get_score()
{
	return score;
}



void ActorPlayer::add_to_score(int points)
{
	score += points;
}



void ActorPlayer::set_score(long int points)
{
	score = points;
}



bool ActorPlayer::is_alive()
{
	return !dead;
}



void ActorPlayer::set_weapon_power(int weapon, int power)
{
	weapon_power[weapon] = power;
}



int ActorPlayer::get_current_weapon()
{
	return current_weapon;
}



char *ActorPlayer::get_weapon_name(int weapon)
{
	if (weapon > 0 && weapon <= NUM_WEAPONS)
		return weapon_name[weapon];
	return "";
}



int ActorPlayer::get_weapon_power(int weapon)
{
	if (weapon > 0 && weapon <= NUM_WEAPONS)
		return weapon_power[weapon];
	return 0;
}

