/***************************************************************************
                                 actor.h
                             -------------------
    begin                : Thu May 25 2000
    copyright            : (C) 2000 by Bytopia Software
    email                : lancekt@wam.umd.edu
                             -------------------
	The base class representing most in-game objects.
	(Note to self: will this need a copy constructor?  Are we ever going to
	copy them?)
	TODO:	-	Add some way for an actor to return a signal if it becomes out of
					a certain scope (like, for example, the screen)
 ***************************************************************************/

#ifndef ACTOR_H
#define ACTOR_H

#include <iostream>
#include "display.h"
#include "mixer.h"
#include "anim.h"
#include "behavior.h"
#include "boundingbox.h"



class ActorList;	// forward definition



class Actor {
	private:
		int x, y;
		int z;			// the actor's z-order; used for placing it in an actorlist
		int collision_damage;
		bool hits_enemies, hits_players;
		Anim *anim;
		Behavior *behavior;
		BoundingBox bounding_box;
		BoundingBox scope;
		bool dies_on_leaving_scope;
		static Display *display;
		static Mixer *mixer;
		ActorList *actorlist;		// reference to container, so the actor can directly modify the list
														// I'm not sure if I'm technically allowed to do this...
	protected:
		char *type;
	public:
	
		struct ActorSignal {
			char *signal;
			Actor *source;
		};

		Actor();
		virtual ~Actor();
		
		static void bind_display(Display *new_display);
		static void bind_mixer(Mixer *new_mixer);
		void set_actorlist(ActorList *new_actorlist);
		ActorList *get_actorlist();
		
		virtual ActorSignal update();
		
		virtual void paint();
		
		virtual ActorSignal collided();
		virtual void get_signal(ActorSignal signal);
		
		void set_position(const int new_x, const int new_y);
		void set_z(const int new_z);
		void set_anim(Anim *new_anim);
		void set_behavior(Behavior *new_behavior);
		
		Behavior *get_behavior();
		Anim *get_anim();
		char *get_current_frame_tag();
		static Display *get_display();
		Mixer *get_mixer();
		int get_x();
		int get_y();
		int get_z();
		void set_damage_types(bool players, bool enemies);
		bool damages_enemies();
		bool damages_players();
		char *get_type();
		
		bool check_collision(const Actor &actor);
		void set_collision_damage(int damage);
		int get_collision_damage();
		
		BoundingBox get_bounding_box();		
		void set_scope(int x1, int y1, int x2, int y2);
		void set_death_on_leaving_scope(bool value);
		bool get_death_on_leaving_scope();
		BoundingBox get_scope();

		void update_bounding_box();
				
		void dump_bounding_box(std::ostream &out);
	protected:
		void update_base();
};






#endif

