
#include "actorlist.h"



ActorList::ActorList()
{
	node_count = 0;
	first = NULL;
}



ActorList::~ActorList()
{
	while (first != NULL)
		remove_node(first);
}



void ActorList::update()
{
	node *walk = first, *temp;
	Actor::ActorSignal signal;
	bool actor_removed = false;
	
	while (walk != NULL) {
		actor_removed = false;
		
		check_collisions(walk);
		
		signal = walk->actor->update();
		if (strcmp(signal.signal, "suicide") == 0) {
			temp = walk;
			walk = walk->next;
			remove_node(temp);
			actor_removed = true;
		}
		if (!actor_removed) walk = walk->next;
	}
}



void ActorList::paint()
{
	node *walk = first;
	
	while (walk != NULL) {
		walk->actor->paint();
		walk = walk->next;
	}
}
		



void ActorList::add_node(node *new_node)
{
	node *walk = first;
	bool added = false;
	
	if (new_node != NULL) {
		if (first == NULL) {
			first = new_node;
		} else if (new_node->actor->get_z() < first->actor->get_z()) {
			new_node->next = first;
			first = new_node;
		} else {
			while ((walk->next != NULL) && (added == false)) {
				if (new_node->actor->get_z() < walk->next->actor->get_z()) {
					new_node->next = walk->next;
					walk->next = new_node;
					walk = walk->next;
					added = true;
				}
				if (added == false) walk = walk->next;
			}
			if (walk->next == NULL) {
				walk->next = new_node;
			}
		}
	} else {
		std::cerr << "WARNING: tried to add a NULL node to an actorlist" << std::endl;
	}
}



void ActorList::add_actor(Actor *actor)
{
	node *new_node = new node;
	new_node->actor = actor;
	new_node->exists_outside_list = false;
	new_node->next = NULL;
	new_node->actor->set_actorlist(this);
	add_node(new_node);
	node_count++;
}



void ActorList::register_actor(Actor *actor)
{
	node *new_node = new node;
	new_node->actor = actor;
	new_node->exists_outside_list = true;
	new_node->next = NULL;
	new_node->actor->set_actorlist(this);
	add_node(new_node);
	node_count++;
}



void ActorList::remove_node(node *dead_node)
{
	node *walk = first, *temp;

	if (first == dead_node) {
		first = walk->next;
		deallocate_node(walk);
	} else {
		if (walk != NULL)
			while (walk->next != NULL) {
				if (walk->next == dead_node) {
					temp = walk->next;
					walk->next = temp->next;
					deallocate_node(temp);
				} else {
					walk = walk->next;
				}
			}
	}
	node_count--;
}
			
		




void ActorList::deallocate_node(node *dead_node)
{
	if (dead_node != NULL) {
		if (dead_node->exists_outside_list == false)
			delete dead_node->actor;
		delete dead_node;
	}
}



void ActorList::check_collisions(node *check_node)
{
	node *walk = check_node->next;
	Actor::ActorSignal signal;
	
	check_node->actor->update_bounding_box();
	
	while (walk != NULL) {
		walk->actor->update_bounding_box();
		if (check_node->actor->get_bounding_box().collides_with(walk->actor->get_bounding_box())) {
			signal = check_node->actor->collided();
			walk->actor->get_signal(signal);
			signal = walk->actor->collided();
			check_node->actor->get_signal(signal);
		}
		walk = walk->next;
	}
}



void ActorList::debug_dump(std::ostream &out)
{
	node *walk = first;
	
	while (walk != NULL) {
		out << walk->actor->get_type() << ": " << walk->actor->get_z() << std::endl;
		walk = walk->next;
	}
}



int ActorList::get_actor_count()
{
	return node_count;
}

