
#include "anim.h"



Display *Anim::display = NULL;



// Constructor.  Initialize everything.
Anim::Anim()
{
	first = NULL;
	current = NULL;
	hash = NULL;
	to_next_frame = 0;
	frame_count = 0;
	looping = true;
	at_end_of_anim = false;
}



// Destructor.  Destroy the linked list.
Anim::~Anim()
{
	clear();
}



// Change whether the animation loops
void Anim::set_looping(bool value)
{
	looping = value;
}



// Empty the linked list.
void Anim::clear()
{
	while (first != NULL) {
		remove_first_node();
	}
	current = NULL;
}



// Update the animation; move to the next frame if there is one.  If next
// frame is empty, loop to beginning if looping is true, otherwise stay
// on last frame.
void Anim::update()
{
	if (current == NULL) {
		// updated null frame; let this go.
	} else {
		to_next_frame++;
		if (to_next_frame >= current->duration) {
			if (current->next == NULL) {
				if (looping == true)
					current = first;
				else
					at_end_of_anim = true;
			} else {
				advance_node();
			}
			to_next_frame = 0;
		}
	}
}



// Resets the current frame to the beginning.
void Anim::reset()
{
	current = first;
	at_end_of_anim = false;
}



// Add a frame to the end of the animation.
void Anim::add_frame(char *tag, int new_duration)
{
	node *new_node = new node;
	new_node->tag = new char[strlen(tag) + 1];
	frame_count++;
	strcpy(new_node->tag, tag);
	new_node->duration = new_duration;
	new_node->offsets.set(0, 0, 0, 0);
	new_node->next = NULL;
	add_node(new_node);
}



// Add a frame to the end of the animation.
void Anim::add_frame(char *tag, int new_duration, int x1, int y1, int x2, int y2)
{
	node *new_node = new node;
	new_node->tag = new char[strlen(tag) + 1];
	strcpy(new_node->tag, tag);
	new_node->duration = new_duration;
	new_node->offsets.set(x1, y1, x2, y2);
	new_node->next = NULL;
	add_node(new_node);
}



// Retrieve a pointer to the current frame's tag string.
char * Anim::get_current_frame()
{
	if (current != NULL)
		return current->tag;
	else
		return NULL;
}



BoundingBox Anim::get_current_frame_offsets()
{
	return current->offsets;
}



// Add a node to the list.  (Private.)
void Anim::add_node(node *new_node)
{
	node *walk = first;
	
	frame_count++;
	
	if (first == NULL) {
		first = new_node;
		current = new_node;
	} else {
		while (walk->next != NULL)
			walk = walk->next;
		walk->next = new_node;
	}
}



// Remove a node from the list.  (Private.)
void Anim::remove_first_node()
{
	node *temp = first;
	
	frame_count--;
	
	if (first != NULL) {
		if (current == temp) current = first;
		first = first->next;
		delete [] temp->tag;
		delete temp;
	}
}



// Move to the next node in the list if available.  (Private.)
void Anim::advance_node()
{
	if (current != NULL)
		current = current->next;
}



bool Anim::at_end()
{
	if (at_end_of_anim)
		return true;
	return false;
}



// Dump some debug information to an std::ostream.
void Anim::debug_dump(std::ostream &out)
{
	node *walk = first;
	
	out << "Nodes from start: ";
	while (walk != NULL) {
		out << walk->tag << " ";
		walk = walk->next;
	}
	out << std::endl << "Nodes from current: ";
	walk = current;
	while (walk != NULL) {
		out << walk->tag << " ";
		walk = walk->next;
	}
	out << std::endl;
}



void Anim::add_hash()
{
	if (first != NULL && display != NULL) {
		hash = display->get_hash_node(first->tag);
	} else {
		std::cerr << "WARNING: tried to hash an anim without a first frame";
	}
}



void Anim::bind_display(Display *new_display)
{
	display = new_display;
}



ImageCache::node *Anim::get_hash()
{
	return hash;
}

