/***************************************************************************
                                 anim.h
                             -------------------
    begin                : Thu May 25 2000
    copyright            : (C) 2000 by Bytopia Software
    email                : lancekt@wam.umd.edu
                             -------------------
	The animation class for Angel Wars.  A linked list of frames that can only
	be added to at the end.  (Meaning in effect that existing animations cannot
	be modified once they are loaded.  This behavior should be reconsidered
	when I have time.)  The data of each frame is actually only a string, which
	is a tag that can be passed to a Display and refers to an actual bitmap in
	the ImageCache bound to the Display.
	To set up an Anim, call add_frame() in the order the frames should be
	played.  Set the looping property to determine whether the animation is
	(you guessed it) looping.
 ***************************************************************************/

#ifndef ANIM_H
#define ANIM_H

#include <iostream>

#include "display.h"
#include "boundingbox.h"



class Anim {
	private:
		struct node {
			char *tag;							// a frame's string tag
			int duration;						// the duration of a frame
			BoundingBox offsets;
			node *next;							// pointer to next frame in list
		};
		node *first, *current;		// pointers to the first frame and the current frame.
		
		bool looping;							// if true, animation will loop
		bool at_end_of_anim;			// if non-looping, set when end reached
		
		int to_next_frame;				// a counter
		int frame_count;
		
		static Display *display;
		
		ImageCache::node *hash;
		
	public:
		Anim();
		~Anim();
		
		void clear();
		void update();
		
		void reset();
		
		void set_looping(bool value);
		bool at_end();

		char *get_current_frame();
		BoundingBox get_current_frame_offsets();		
		void add_frame(char *tag, int new_duration);
		void add_frame(char *tag, int new_duration, int x1, int y1 = 0, int x2 = 0, int y2 = 0);
		
		static void bind_display(Display *new_display);
		
		void add_hash();
		ImageCache::node *get_hash();
		
		void debug_dump(std::ostream &out);
	private:
		void add_node(node *new_node);
		void remove_first_node();
		void advance_node();
};

#endif

