
#include "audiocache.h"



// Initialize the first node of the list to NULL.
AudioCache::AudioCache()
{
	first = NULL;
}



// Clear the list before destruction.
AudioCache::~AudioCache()
{
	clear();
}



// Call remove_node for each item on the list.  (It could be more
// efficient to walk it normally instead, but I'm too lazy.)
void AudioCache::clear()
{
	while (first != NULL)
		remove_node(first);
}



// Load a sound from a file into the linked list.
void AudioCache::add_sound_from_file(char *file, char *new_tag)
{
	node *new_sound = new node;
	char *allocated_tag = new char[strlen(new_tag) + 1];

	strcpy(allocated_tag, new_tag);
		
	new_sound->data = Mix_LoadWAV(file);
	
	if (new_sound->data == NULL) {
		std::cerr << "FATAL: Unable to load wave: " << file << std::endl;
		exit(0);
	} else {
		// okie doke
	}
	new_sound->tag = allocated_tag;
	new_sound->next = NULL;
	
	add_node(new_sound);
}



// remove a sound
void AudioCache::remove_sound(char *dead_tag)
{
	node *dead_node = find_node(dead_tag);
		
	if (dead_node != NULL) {
		remove_node(dead_node);
	}
}



// Return a pointer to the sound
Mix_Chunk *AudioCache::get_sound(char *search_tag)
{
	node *search_node = find_node(search_tag);
	
	if (search_node == NULL) {
		std::cerr << "WARNING: Sound " << search_tag << " not found in cache; "
			<< "returning NULL instead." << std::endl;
		return NULL;
	} else {
		return search_node->data;
	}
}



// Add a node to the list.  (Private.)
void AudioCache::add_node(node *new_node)
{
	node *walk = first;

	if (first == NULL) {
		first = new_node;
	} else {
		while (walk->next != NULL)
			walk = walk->next;
		walk->next = new_node;
	}
}



// Remove a node from the list.  (Private.)
void AudioCache::remove_node(node *dead_node)
{
	node *walk = first;
	
	if (first == dead_node) {
		first = first->next;
		delete walk;
	} else {
		while ((walk->next != dead_node) && (walk->next != NULL))
			walk = walk->next;
		if (walk->next == NULL) {
			std::cerr << "WARNING: tried to remove a node that isn't in the list."
				<< std::endl;
		} else {
			walk->next = walk->next->next;
			Mix_FreeChunk(dead_node->data);
			delete [] dead_node->tag;
			delete dead_node;
		}
	}
}



// Locate a node, by tag.  (Private.)
AudioCache::node *AudioCache::find_node(char *search_tag)
{
	node *walk = first;
	
	if (strcmp(first->tag, search_tag) == 0)
		return first;
	while (walk->next != NULL) {
		walk = walk->next;
		if (strcmp(walk->tag, search_tag) == 0)
			return walk;
	}
	return NULL;
}



void AudioCache::load_sounds_from_list_file(char *filename)
{
	std::ifstream file;
	char sound_file[250];
	char sound_tag[250];
	
	file.open(filename);
	
	if (file) {
		while (!file.eof()) {
			file >> std::ws;
			file >> sound_file >> std::ws >> sound_tag >> std::ws;
			add_sound_from_file(sound_file, sound_tag);
		}
	}
}



// Dump debug information (currently the contents of the list) to an std::ostream.
void AudioCache::debug_dump(std::ostream &out)
{
	node *walk = first;

	out << "cache:" << std::endl;
	while (walk != NULL) {
		out << walk->tag << std::endl;
		walk = walk->next;
	}
}






