
#include "behavior-from-keyboard.h"



BehaviorFromKeyboard::BehaviorFromKeyboard()
{
	event_handler = NULL;
	x_impulse = BEHAVIOR_FROM_KEYBOARD_DEFAULT_X_IMPULSE;
	y_impulse = BEHAVIOR_FROM_KEYBOARD_DEFAULT_Y_IMPULSE;
	x_inertia = BEHAVIOR_FROM_KEYBOARD_DEFAULT_X_INERTIA;
	y_inertia = BEHAVIOR_FROM_KEYBOARD_DEFAULT_Y_INERTIA;
	dx_max = BEHAVIOR_FROM_KEYBOARD_DEFAULT_DX_MAX;
	dy_max = BEHAVIOR_FROM_KEYBOARD_DEFAULT_DY_MAX;
	x_min = BEHAVIOR_FROM_KEYBOARD_DEFAULT_XMIN;
	y_min = BEHAVIOR_FROM_KEYBOARD_DEFAULT_YMIN;
	x_max = BEHAVIOR_FROM_KEYBOARD_DEFAULT_XMAX;
	y_max = BEHAVIOR_FROM_KEYBOARD_DEFAULT_YMAX;
}



void BehaviorFromKeyboard::bind_event_handler(Event *new_event)
{
	event_handler = new_event;
}



void BehaviorFromKeyboard::set_impulse(float new_x, float new_y)
{
	x_impulse = new_x;
	y_impulse = new_y;
}



void BehaviorFromKeyboard::set_inertia(float new_x, float new_y)
{
	x_inertia = new_x;
	y_inertia = new_y;
}



void BehaviorFromKeyboard::set_maxima(float new_x, float new_y)
{
	dx_max = new_x;
	dy_max = new_y;
}



void BehaviorFromKeyboard::set_bounds(int new_xmin, int new_ymin, int new_xmax, int new_ymax)
{
	x_min = new_xmin;
	y_min = new_ymin;
	x_max = new_xmax;
	y_max = new_ymax;
}



char *BehaviorFromKeyboard::update(int &ret_x, int &ret_y)
{
	char *retval = "";
	
	//std::cerr << get_dx();

	if (event_handler == NULL) {
		std::cerr << "WARNING: tried to update BehaviorFromKeyboard before binding event handler" << std::endl;;
		return retval;
	}
	
	if (event_handler->key()[KEY_LCTRL])
		retval = "fire";	
	if (event_handler->key()[KEY_LEFT])
		set_dx(get_dx() - x_impulse);
	if (event_handler->key()[KEY_RIGHT])
		set_dx(get_dx() + x_impulse);
	if (event_handler->key()[KEY_UP])
		set_dy(get_dy() - y_impulse);
	if (event_handler->key()[KEY_DOWN])
		set_dy(get_dy() + y_impulse);
	if (event_handler->key()[KEY_COMMA])
		retval = "next_weapon";
	if (event_handler->key()[KEY_PERIOD])
		retval = "prev_weapon";
	if (get_dx() > 0) {
		set_dx(get_dx() - x_inertia);
		if (get_dx() > dx_max)
			set_dx(dx_max);
		if (get_dx() < 0)
			set_dx(0);
	}
	if (get_dx() < 0) {
		set_dx(get_dx() + x_inertia);
		if (get_dx() < -dx_max)
			set_dx(-dx_max);
		if (get_dx() > 0)
			set_dx(0);
	}
	if (get_dy() > 0) {
		set_dy(get_dy() - y_inertia);
		if (get_dy() > dy_max)
			set_dy(dy_max);
		if (get_dy() < 0)
			set_dy(0);
	}
	if (get_dy() < 0) {
		set_dy(get_dy() + y_inertia);
		if (get_dy() < -dy_max)
			set_dy(-dx_max);
		if (get_dy() > 0)
			set_dy(0);
	}
	update_linear();
	if (get_x() < x_min) {
		set_x(x_min);
		set_dx(0);
	} else if (get_x() > x_max) {
		set_x(x_max);
		set_dx(0);
	}
	if (get_y() < y_min) {
		set_y(y_min);
		set_dy(0);
	} else if (get_y() > y_max) {
		set_y(y_max);
		set_dy(0);
	}
	ret_x = get_x();
	ret_y = get_y();

	return retval;
}

