
#include "display.h"




bool Display::already_instantiated = false;




Display::Display(char *title)
{
	if (already_instantiated == true) {
		std::cerr << "FATAL: tried to instantiate more than one Display" << std::endl;
		exit(0);
	}
	
	already_instantiated = true;

	SDL_Init(SDL_INIT_VIDEO);
  SDL_WM_SetCaption(title, NULL);
	
	w = 0;
	h = 0;
	depth = 0;
	cache = NULL;
	screen = NULL;
	//buffer = NULL;
}



Display::~Display()
{
	//if (buffer != NULL) SDL_FreeSurface(buffer);
	if (screen != NULL) SDL_FreeSurface(screen);
	SDL_Quit();
}



bool Display::set_mode(int x, int y, int color_depth)
{
	screen = SDL_SetVideoMode(x, y, color_depth, SDL_DOUBLEBUF|SDL_HWSURFACE);
	w = x;
	h = y;
	depth = color_depth;
	//if (buffer != NULL) SDL_FreeSurface(buffer);
	//buffer = SDL_CreateRGBSurface(SDL_HWSURFACE, x, y, color_depth, 0, 0, 0, 0);
	//if (buffer == NULL) {
	//	std::cerr << "FATAL: Unable to allocate a backbuffer." << std::endl;
	//	exit(0);
	//}
	//SDL_FillRect(buffer, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
	return true;
}



void Display::start_new_frame()
{
	//SDL_FillRect(buffer, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
}




void Display::paint_current_frame()
{
	//SDL_BlitSurface(buffer, NULL, screen, NULL);
	SDL_Flip(screen);
}



void Display::bind_cache(ImageCache *new_cache)
{
	cache = new_cache;
}



void Display::paint_sprite(int x, int y, char *tag, ImageCache::node *hash)
{
	SDL_Surface *pic;
	SDL_Rect dest = {x, y, 0, 0};
	int off_x, off_y;
	
	if (cache != NULL) {
		if (tag != NULL) {
			pic = cache->get_image(tag, off_x, off_y, hash);
			if (pic != NULL) {
				dest.w = pic->w;
				dest.h = pic->h;
				dest.x += off_x;
				dest.y += off_y;
				SDL_BlitSurface(pic, NULL, screen, &dest);
			} else {
				std::cerr << "WARNING: paint_sprite() couldn't obtain image from cache." << std::endl;
			}
		} else {
			// tried to paint null sprite; just let it go
		}
	} else {
		std::cerr << "WARNING: paint_sprite() called before cache has been bound." << std::endl;
	}
}



BoundingBox Display::get_sprite_dimensions(char *tag)
{
	BoundingBox ret;
	if (cache != NULL) {
		ret = cache->get_image_dimensions(tag);
	} else {
		std::cerr << "WARNING: get_sprite_dimensions called before cache bound" << std::endl;
		ret.set(0, 0, 0, 0);
	}
	return ret;
}	



int Display::get_width()
{
	return w;
}



int Display::get_height()
{
	return h;
}



int Display::get_color_depth()
{
	return depth;
}



SDL_Surface *Display::get_buffer()
{
	return screen;
}



ImageCache::node *Display::get_hash_node(char *tag)
{
	return cache->find_node(tag);
}

