
#include "enemy-bat.h"



EnemyBat::EnemyBat()
{
	type = "EnemyBat";
	
	target = NULL;
	needs_to_fire = false;

	Anim *bat_anim = new Anim;
	Anim *bat_dies = new Anim;
	Anim *bat_fires = new Anim;
	
	bat_anim->set_looping(true);
	bat_anim->add_frame("batattack00", 1);
	bat_anim->add_hash();
	bat_dies->set_looping(false);
	bat_dies->add_frame("batdeath00", 5);
	bat_dies->add_frame("batdeath01", 5);
	bat_dies->add_frame("batdeath02", 5);
	bat_dies->add_frame("batdeath03", 5);
	bat_dies->add_frame("batdeath04", 5);
	bat_dies->add_frame("batdeath05", 5);
	bat_dies->add_hash();
	bat_fires->set_looping(false);
	bat_fires->add_frame("batattack01", 5);
	bat_fires->add_frame("batattack02", 5);
	bat_fires->add_frame("batattack03", 5);
	bat_fires->add_frame("batattack02", 5);
	bat_fires->add_frame("batattack01", 5);
	bat_fires->add_hash();
	
	set_idle_anim(bat_anim);
	set_death_anim(bat_dies);
	fire_anim = bat_fires;
	set_anim(bat_anim);
	fire_anim = bat_fires;
	
	set_collision_damage(1);
	set_hp(20);
	set_damage_types(true, false);
	set_hurt_types(false, true);
	
	set_death_behavior(NULL);
	set_scope(-20, -80, get_display()->get_width() + 80,
	get_display()->get_height() + 80);
	set_death_on_leaving_scope(true);
	set_point_value(75);
	
	sound_fired = false;
}



EnemyBat::~EnemyBat()
{
	if (strcmp(type, "EnemyBat") == 0) {
		if (get_idle_anim() != NULL)
			delete get_idle_anim();
		if (get_death_anim() != NULL)
			delete get_death_anim();
		if (fire_anim != NULL)
			delete fire_anim;
		if (get_behavior() != NULL)
			delete get_behavior();
		if (get_death_behavior() != NULL)
			delete get_death_behavior();
	}
}



void EnemyBat::set_target(Actor *new_target)
{
	target = new_target;
}



Actor::ActorSignal EnemyBat::update()
{
	ActorSignal retsignal = {"", this};
	char *rec_sig;
	int temp_x, temp_y;

	if (get_hp() <= 0) {
		if (!sound_fired) {
			get_mixer()->play_sound("batcall");
			sound_fired = true;
		}
		dead = true;
		set_anim(get_death_anim());
		if (get_death_behavior() != NULL) {
			if (get_behavior() != NULL) delete get_behavior();
			set_behavior(get_death_behavior());
			set_death_behavior(NULL);
		}
		if (get_death_anim()->at_end()) {
			// won't commit suicide until out of scope
			//retsignal.signal = "suicide";
			//retsignal.source = this;
		}
	}
	if (get_death_on_leaving_scope() && !get_scope().contains(get_bounding_box())) {
		retsignal.signal = "suicide";
		retsignal.source = this;
	}
	if (get_behavior() != NULL) rec_sig = get_behavior()->update(temp_x, temp_y);
	set_position(temp_x, temp_y);
	//if (strcmp(rec_sig, "fire") == 0)
		fire_blobbie(rec_sig);
	if (get_anim() != NULL) {
		get_anim()->update();
	}
	return retsignal;
}



/*void EnemyBat::get_signal(ActorSignal signal)
{
	if ((strcmp(signal.signal, "hurt") == 0) && !dead)
		if (signal.source->damages_players() && gets_hurt_as_player()) {
			hurt(signal.source->get_collision_damage());
		} else if (signal.source->damages_enemies() && gets_hurt_as_enemy()) {
			hurt(signal.source->get_collision_damage());
		}		
}*/



void EnemyBat::fire_blobbie(char *rec_sig)
{
	if (!dead) {
		if ((get_anim() == fire_anim) && (fire_anim->at_end()))
			set_anim(get_idle_anim());
		if (strcmp(rec_sig, "fire") == 0) {
			fire_anim->reset();
			set_anim(fire_anim);
			needs_to_fire = true;
		}
		if ((get_anim() == fire_anim) && (strcmp(fire_anim->get_current_frame(), "batattack03") == 0) && (needs_to_fire == true)) {
			needs_to_fire = false;
			BehaviorFollow *be = new BehaviorFollow;
			be->set_target(target);
			Enemy *shot = new Enemy;
			Anim *shot_anim = new Anim;
			Anim *shot_d_anim = new Anim;
			shot_anim->set_looping(true);
			shot_d_anim->set_looping(false);
			shot_anim->add_frame("star00", 8);
			shot_anim->add_frame("star01", 8);
			shot_anim->add_hash();
			shot_d_anim->add_frame("smallpuff00", 3);
			shot_d_anim->add_frame("smallpuff01", 3);
			shot_d_anim->add_frame("smallpuff02", 3);
			shot_d_anim->add_frame("smallpuff03", 3);
			shot_d_anim->add_hash();
			
			shot->set_idle_anim(shot_anim);
			shot->set_death_anim(shot_d_anim);
			//shot->set_anim(shot_anim);
			shot->set_position(get_x() + 9, get_y() + 9);
		
			shot->set_collision_damage(5);
			shot->set_hp(2);
			shot->set_damage_types(true, false);
			shot->set_hurt_types(false, true);
			
			shot->set_behavior(be);
			shot->set_point_value(5);
		
			shot->set_scope(-15, -25, get_display()->get_width() + 25,
				get_display()->get_height() + 25);
			shot->set_death_on_leaving_scope(true);
			
			get_actorlist()->add_actor(shot);	
		}
	}
}















