
#include "enemy-morte.h"



EnemyMorte::EnemyMorte()
{
	type = "EnemyMorte";
	has_spawned_pieces = false;

	Anim *morte_anim = new Anim;
	Anim *morte_dies = new Anim;
	morte_anim->set_looping(true);
	morte_dies->set_looping(false);
	morte_anim->add_frame("skull00", 1);
	morte_anim->add_hash();
	morte_dies->add_frame("skulldeath00", 2);
	morte_dies->add_frame("skulldeath01", 2);
	morte_dies->add_frame("skulldeath02", 2);
	morte_dies->add_frame("skulldeath03", 5);
	morte_dies->add_frame("skulldeath04", 5);
	morte_dies->add_frame("skulldeath05", 5);
	morte_dies->add_frame("skulldeath06", 5);
	morte_dies->add_hash();
	set_collision_damage(10);
	set_hp(2);
	set_damage_types(true, false);
	set_hurt_types(false, true);
	Behavior *be_death = new Behavior;
	set_death_behavior(be_death);
	set_idle_anim(morte_anim);
	set_death_anim(morte_dies);
	set_anim(morte_anim);
	set_scope(-20, -80, get_display()->get_width() + 80,
	get_display()->get_height() + 80);
	set_death_on_leaving_scope(true);
	set_point_value(25);
	
	sound_fired = false;
}



EnemyMorte::~EnemyMorte()
{
	if (strcmp(type, "EnemyMorte") == 0) {
		if (get_idle_anim() != NULL)
			delete get_idle_anim();
		if (get_death_anim() != NULL)
			delete get_death_anim();
		if (get_behavior() != NULL)
			delete get_behavior();
		if (get_death_behavior() != NULL)
			delete get_death_behavior();
	}
}



Actor::ActorSignal EnemyMorte::update()
{
	ActorSignal retsignal = {"", this};

	if (get_hp() > 0) set_anim(get_idle_anim());
	if (get_hp() <= 0) {
		if (strcmp(get_anim()->get_current_frame(), "skulldeath04") == 0)
			if (has_spawned_pieces == false) {
				spawn_pieces();
				has_spawned_pieces = true;
			}
		if (!sound_fired) {
			sound_fired = true;
			get_mixer()->play_sound("crumble");
		}
		dead = true;
		set_anim(get_death_anim());
		if (get_death_behavior() != NULL) {
			if (get_behavior() != NULL) delete get_behavior();
			set_behavior(get_death_behavior());
			set_death_behavior(NULL);
		}
		if (get_death_anim()->at_end()) {
			retsignal.signal = "suicide";
			retsignal.source = this;
		}
	}
	if (get_death_on_leaving_scope() && !get_scope().contains(get_bounding_box())) {
		retsignal.signal = "suicide";
		retsignal.source = this;
	}
	update_base();
	return retsignal;
}



// Morte dies; spawn some Morte bits!
void EnemyMorte::spawn_pieces()
{
	Actor *piece;
	BehaviorQuadratic *be;
	Anim *anim;

	piece = new Actor;
	be = new BehaviorQuadratic;
	be->set_d2y(.2);
	be->set_dx(-3);
	anim = new Anim;
	anim->set_looping(true);
	anim->add_frame("skullpiece00", 1);
	piece->set_position(get_x(), get_y());
	piece->set_anim(anim);
	piece->set_behavior(be);
	piece->set_scope(-30, -80, get_display()->get_width() + 30,
		get_display()->get_height() + 30);
	piece->set_death_on_leaving_scope(true);
	get_actorlist()->add_actor(piece);
	piece = new Actor;
	be = new BehaviorQuadratic;
	be->set_d2y(.2);
	be->set_dx(-3.5);
	be->set_dy(-3);
	anim = new Anim;
	anim->set_looping(true);
	anim->add_frame("skullpiece01", 1);
	piece->set_position(get_x(), get_y());
	piece->set_anim(anim);
	piece->set_scope(-30, -80, get_display()->get_width() + 30,
		get_display()->get_height() + 30);
	piece->set_death_on_leaving_scope(true);
	piece->set_behavior(be);
	get_actorlist()->add_actor(piece);
	piece = new Actor;
	be = new BehaviorQuadratic;
	anim = new Anim;
	anim->set_looping(true);
	anim->add_frame("skullpiece02", 1);
	be->set_d2y(.2);
	be->set_dx(3);
	be->set_dy(-3.5);
	piece->set_position(get_x(), get_y());
	piece->set_anim(anim);
	piece->set_behavior(be);
	piece->set_scope(-30, -80, get_display()->get_width() + 30,
		get_display()->get_height() + 30);
	piece->set_death_on_leaving_scope(true);
	get_actorlist()->add_actor(piece);
	piece = new Actor;
	be = new BehaviorQuadratic;
	anim = new Anim;
	anim->set_looping(true);
	anim->add_frame("skullpiece03", 1);
	be->set_d2y(.2);
	be->set_dx(2.5);
	be->set_dy(-2);
	piece->set_position(get_x(), get_y());
	piece->set_anim(anim);
	piece->set_behavior(be);
	piece->set_scope(-30, -80, get_display()->get_width() + 30,
		get_display()->get_height() + 30);
	piece->set_death_on_leaving_scope(true);
	get_actorlist()->add_actor(piece);
}





