/***************************************************************************
                                 imagecache.h
                             -------------------
    begin                : Thu May 25 2000
    copyright            : (C) 2000 by Bytopia Software
    email                : lancekt@wam.umd.edu
                             -------------------
	A class used for storing images.
	This is platform-dependent, in this case using the SDL library.
	allegro_init() MUST be called before any ImageCache methods are
	invoked.

	An ImageCache is a dynamically allocated structure containing images
	referenced by string tags.

	TODO: - Implement a function to load a list of images to be inserted
	        into the cache from a file.
	      - Make the insert function add the new item into the beginning of
	        the list.  This is more efficient, especially since order doesn't
	        matter.
 ***************************************************************************/

#ifndef IMAGECACHE_H
#define IMAGECACHE_H

#include <iostream>
#include <fstream>
#include <string.h>
#include <SDL/SDL.h>
#include "SDL_image.h"

#include "boundingbox.h"



class ImageCache {
	public:
		struct node {
			SDL_Surface *data;
			char *tag;
			node *next;
			int paint_offset_x, paint_offset_y;
		};
	private:
		node *first;
	public:
		ImageCache();
		~ImageCache();
		void clear();
		void add_image_from_file(char *file, char *new_tag);
		void add_image_from_file(char *file, char *new_tag, int off_x, int off_y);
		void remove_image(char *dead_tag);
		SDL_Surface *get_image(char *search_tag, int &off_x, int &off_y, node *hash = NULL);
		BoundingBox get_image_dimensions(char *search_tag);
		
		void load_images_from_list_file(char *filename);

		void debug_dump(std::ostream &out);
		
		node *find_node(char *search_tag, node *hash = NULL);
	private:
		void add_node(node *new_node);
		void remove_node(node *dead_node);
};

#endif

