/***************************************************************************
                                 Angel Wars
                             -------------------
    begin                : Fri May 26 2000
    copyright            : (C) 2000 by Bytopia Software
    email                : lancekt@wam.umd.edu
                             -------------------
	The main program--the gizmo that binds all the pieces into one incoherent
	lump of programming goo.  This is probably only temporary, as it is a
	really crappy way to glue things together.
 ***************************************************************************/


#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
#include <SDL/SDL.h>

#include "actor-player.h"
#include "enemy-morte.h"
#include "enemy-bat.h"
#include "enemy-slave.h"
#include "enemy-fatboy.h"
#include "bullet-firebolt.h"
#include "actorlist.h"
#include "display.h"
#include "imagecache.h"
#include "background.h"
#include "behavior-periodic.h"
#include "behavior-sinewave.h"
#include "audiocache.h"

#include "font.h"
#include "SDL_image.h"
#include "mixer.h"

#include "main-helper.h"
#include "main-menu.h"

#define GAME_SPEED 20

volatile int update_count = 0;

Uint32 callback(Uint32 interval)
{
	update_count++;
	return 20;
}



bool run_level(char *level_file, Display *display, Mixer *mixer, ImageCache *cache, AudioCache *audio)
{
	Background::bind_display(display);
	float position = 0;
	bool level_done = false;
	char buf[100];
	ActorList list;
	Event event_handler;
	Background bg1, bg2;
	float bg1_speed, bg2_speed;
	int file_position = 0;
	int level_end_position;
	bool victory = true;

	int frames_skipped = 0;	
	long total_frames = 0, total_skips = 0;
	float skips_per_frame = 0;
	
	SDL_Surface *font = IMG_Load("/usr/share/games/angelwars/images/fonts/smstone.png");
	InitFont(font);

	display->bind_cache(cache);
	mixer->bind_cache(audio);
		
	std::ifstream file(level_file, std::ios::in);
	if (!file) {
		std::cerr << "FATAL: unable to load level " << level_file << std::endl;
		std::cerr << "Did you get the angelwars-levels package?" << std::endl;
		exit(0);
	}
	file >> level_end_position;
	file >> std::ws >> buf >> bg1_speed;
	cache->add_image_from_file(buf, buf);
	bg1.set_image(buf);
	file >> std::ws >> buf >> bg2_speed;
	cache->add_image_from_file(buf, buf);
	bg2.set_image(buf);	
	
	file >> std::ws >> buf;
	while (strcmp(buf, "<done>") != 0) {
		cache->load_images_from_list_file(buf);
		file >> std::ws >> buf >> std::ws;
	}
	file >> std::ws >> buf;
	while (strcmp(buf, "<done>") != 0) {
		audio->load_sounds_from_list_file(buf);
		file >> std::ws >> buf >> std::ws;
	}
	
	file >> file_position;
	
	ActorPlayer player;
	BehaviorFromKeyboard *b_from_k = new BehaviorFromKeyboard;
	b_from_k->bind_event_handler(&event_handler);
	b_from_k->set_bounds(-5, 0, 260, 216);
	player.set_behavior(b_from_k);
	player.set_position(0, 100);
	
	
	list.register_actor(&player);
	
	SDL_SetTimer(20, callback);
	
	while (!level_done) {
		while (update_count > 0) {
			event_handler.poll();
			update_count--;
			frames_skipped++;
			if (frames_skipped > 15) {			// if we're skipping too many frames, allow the game to run slower
				update_count = 0;
				break;
			}
			list.update();
			bg1.scroll(bg1_speed);
			bg2.scroll(bg2_speed);
			position += 1.5;
			if (!player.is_alive())
				victory = false;
			if (position >= level_end_position) {
				level_done = true;
			}
			
			if (player.get_score() >= 1500) player.set_weapon_power(1, 2);
			if (player.get_score() >= 3000) player.set_weapon_power(1, 3);
			
			file >> std::ws;
			
			while (!file.eof() && position >= file_position) {
				float param1, param2;
				file >> buf;
				if (strcmp(buf, "spawn_smart_morte") == 0) {
					EnemyMorte *morte = new EnemyMorte;
					BehaviorFollow *smart = new BehaviorFollow;
					file >> param1 >> param2;
					morte->set_position(param1, param2);
					smart->set_position(param1, param2);
					smart->set_target(&player);
					file >> param1 >> param2;
					smart->set_speed_maxima(param2, param2);
					smart->set_impulse(param1, param1);
					morte->set_behavior(smart);
					morte->update();
					list.add_actor(morte);
				}
				if (strcmp(buf, "spawn_dumb_morte") == 0) {
					EnemyMorte *morte = new EnemyMorte;
					BehaviorQuadratic *dumb = new BehaviorQuadratic;
					file >> param1 >> param2;
					morte->set_position(param1, param2);
					dumb->set_position(param1, param2);
					file >> param1 >> param2;
					dumb->set_dx(param1);
					dumb->set_dy(param2);
					file >> param1 >> param2;
					dumb->set_d2x(param1);
					dumb->set_d2y(param2);
					morte->set_behavior(dumb);
					morte->update();
					list.add_actor(morte);
				}
				if (strcmp(buf, "spawn_sine_morte") == 0) {
					EnemyMorte *morte = new EnemyMorte;
					BehaviorSineWave *sine = new BehaviorSineWave;
					file >> param1 >> param2;
					morte->set_position(param1, param2);
					sine->set_position(param1, param2);
					file >> param1;
					sine->set_dx(param1);
					file >> param1 >> param2;
					sine->set_baseline(param1);
					sine->set_impulse(param2);
					morte->set_behavior(sine);
					morte->update();
					list.add_actor(morte);
				}
				if (strcmp(buf, "spawn_slave") == 0) {
					EnemySlave *slave = new EnemySlave;
					BehaviorLinear *be = new BehaviorLinear;
					file >> param1 >> param2;
					slave->set_position(param1, param2);
					be->set_position(param1, param2);
					file >> param1;
					be->set_dx(param1);
					slave->set_behavior(be);
					slave->update();
					list.add_actor(slave);
				}
				if (strcmp(buf, "spawn_fatboy") == 0) {
					EnemyFatboy *fatboy = new EnemyFatboy;
					BehaviorPeriodic *periodic = new BehaviorPeriodic;
					BehaviorSineWave *be = new BehaviorSineWave;
					file >> param1 >> param2;
					fatboy->set_position(param1, param2);
					be->set_position(param1, param2);
					periodic->set_position(param1, param2);
					file >> param1 >> param2;
					be->set_dx(param1);
					periodic->set_period(param2);
					file >> param1 >> param2;
					be->set_baseline(param1);
					be->set_impulse(param2);
					periodic->set_background_behavior(be);
					fatboy->set_behavior(periodic);
					fatboy->update();
					list.add_actor(fatboy);
				}
				if (strcmp(buf, "spawn_bat") == 0) {
					EnemyBat *bat = new EnemyBat;
					bat->set_target(&player);
					BehaviorPeriodic *periodic = new BehaviorPeriodic;
					BehaviorLinear *background = new BehaviorLinear;
					file >> param1 >> param2;
					bat->set_position(param1, param2);
					periodic->set_position(param1, param2);
					background->set_position(param1, param2);
					periodic->set_background_behavior(background);
					file >> param1 >> param2;
					background->set_dx(param1);
					periodic->set_period(param2);
					bat->set_behavior(periodic);
					bat->update();
					list.add_actor(bat);
				}
				if (strcmp(buf, "completed") == 0) {
					if (victory) {
						Actor *actor = new Actor;
						Behavior *be = new Behavior;
						cache->add_image_from_file("/usr/share/games/angelwars/images/menus/completed.png", "completed");
						Anim *temp = new Anim;
						temp->add_frame("completed", 0);
						temp->set_looping(true);
						actor->set_anim(temp);
						actor->set_behavior(be);
						list.add_actor(actor);
					}
				}
				file >> file_position;
			}
			
			if (event_handler.killprogram()) {
				SDL_Quit();
				exit(0);
			}
			if (event_handler.key()[KEY_ESCAPE]) {
				victory = false;
				level_done = true;
			}
			//if (event_handler.key()[KEY_PAUSE]) while (true) {}
		}
		//std::cerr << frames_skipped << " ";
		total_skips += frames_skipped;
		frames_skipped = 0;
		total_frames++;
		char score_string[40];
		char score[20];
		display->start_new_frame();
		bg1.paint();
		bg2.paint();
		list.paint();
		strcpy(score_string, "Score: ");
		longtostr(player.get_score(), score);
		strcat(score_string, score);
		PutString(display->get_buffer(), 0, 225, score_string);
		strcpy(score_string, "Health: ");
		longtostr((unsigned long)player.get_hp(), score);
		strcat(score_string, score);
		PutString(display->get_buffer(), 160 - (TextWidth(score_string) / 2), 225, score_string);
		strcpy(score_string, player.get_weapon_name(player.get_current_weapon()));
		strcat(score_string, " ");
		longtostr((unsigned long)player.get_weapon_power(player.get_current_weapon()), score);
		strcat(score_string, score);
		PutString(display->get_buffer(), (315 - TextWidth(score_string)), 225, score_string);
		longtostr((long)position, score, 1);
		PutString(display->get_buffer(), (315 - TextWidth(score)), 0, score);
		display->paint_current_frame();
		//std::cerr << player.get_weapon_name(player.get_current_weapon());
	}
	
	skips_per_frame = (float)total_skips / total_frames;
	std::cerr << "Average frameskip: " << skips_per_frame << std::endl;
	
	delete cache;
	file.close();

	return victory;		
}



int main()
{
	SDL_Init(SDL_INIT_TIMER);
	
	Display display;
	display.set_mode(320, 240, 32);
	//display.set_mode(1024, 768, 24);
	//exit(0);
	Mixer mixer;
	Actor::bind_display(&display);	
	Actor::bind_mixer(&mixer);
	Anim::bind_display(&display);
	ImageCache *cache = new ImageCache;
	AudioCache *audio = new AudioCache;
	
	cache->load_images_from_list_file("/usr/share/games/angelwars/images/demon.txt");

	menu_result_t result;

	bool victory;
	
	result = show_menu(&display);
	switch (result) {
		case menu_result_quit:
			break;
		case menu_result_play:
			victory = run_level("/usr/share/games/angelwars/levels/level00.txt", &display, &mixer, cache, audio);
			if (victory) {
				std::cout << "Game over - you win!  Sort of" << std::endl;
			} else {
				std::cout << "Game over - you lose." << std::endl;
			}
			break;
		case menu_result_load:
			std::cerr << "Not yet implemented...if you would like to help, please visit" << std::endl;
			std::cerr << "our page at http://sourceforge.net/project/?group_id=6254 and" << std::endl;
			std::cerr << "drop us a note!" << std::endl;
			break;
		case menu_result_options:
			std::cerr << "Not yet implemented...if you would like to help, please visit" << std::endl;
			std::cerr << "our page at http://sourceforge.net/project/?group_id=6254 and" << std::endl;
			std::cerr << "drop us a note!" << std::endl;
			break;
		default:
			break;
	}
	std::cerr << std::endl;
	std::cerr << "Thanks for trying out Angel Wars!  As you can see, there's a lot" << std::endl;
	std::cerr << "of work to be done, and if you'd like to help, visit our web page" << std::endl;
	std::cerr << "at http://sourceforge.net/project/?group_id=6254 and drop us a" << std::endl;
	std::cerr << "note!  We'd especially like someone to do backgrounds and menus" << std::endl;
	std::cerr << "for the game.  And please excuse the hideous main menu.  Dammit," << std::endl;
	std::cerr << "Jim, I'm a programmer, not an artist!  (And Chris has more" << std::endl;
	std::cerr << "important things to work on. ;)" << std::endl;
	
}
